@@ -40,9 +40,11 @@ class NN(object):
         derivative3 (Callable[..., cupy.ndarray]): Derivative of ``loss``
             with respect to predictions (same signature as ``loss``).
         learning_rate (float): SGD step size.
+        scheduler (Scheduler): Learning rate scheduler.
         dtype (cupy.dtype, optional): Floating point dtype for parameters and data.
             Defaults to ``cupy.float32``.
         backup (str): Path for saving the highest performing model during training.
+        initializer (Initializer): Function for initializing weights.
 
     Attributes:
         W (list[cupy.ndarray]): Layer weight matrices; ``W[i]`` has shape
@@ -50,7 +52,7 @@ class NN(object):
   """
 
   def __init__(self, features, architecture, activation, derivative1,
-               resolver, derivative2, loss, derivative3, learning_rate,
+               resolver, derivative2, loss, derivative3, learning_rate, scheduler=None,
                dtype=cp.float32, backup="/tmp/nn.pkl", initializer=None):
     self.architecture = architecture
     self.activation = activation
@@ -59,9 +61,10 @@ class NN(object):
     self.resolver_derivative = derivative2
     self.loss = loss
     self.loss_derivative = derivative3
-    self.learning_rate = learning_rate
+    self.lr = learning_rate
     self.dtype = dtype
     self.backup = backup
+    self.scheduler = scheduler
     self.initializer = initializer
 
     self.W = []
@@ -76,15 +79,19 @@ class NN(object):
       self.W.append(w)
       features = nodes
 
-  def train(self, input, output, epochs=1, batch=0, verbose=False, step=1000, autosave=False):
-    """Train the model using simple SGD.
+  def train(self, input, output, epoch=1, epochs=1, batch=0, verbose=False, step=1000, autosave=False, minloss=[999999999]):
+    """Train the model for one epoch using simple SGD.
+
+        Each epoch is a full pass over the training data.
+        Note that your own external training loop is expected.
 
         Args:
             input (cupy.ndarray | numpy.ndarray): Training inputs of shape
               (n_samples, n_features). If NumPy, it will be moved to device.
             output (cupy.ndarray | numpy.ndarray): Training targets of shape
               (n_samples, n_outputs). If NumPy, it will be moved to device.
-            epochs: Number of SGD steps to run.
+            epoch: Step number of epoch.
+            epochs: Expected number of SGD steps that will be run.
             batch: One of:
                 - ``1``: sample a single example per step (pure SGD)
                 - ``0``: use all samples per step (full batch)
@@ -92,59 +99,62 @@ class NN(object):
             verbose (bool): Print progress to stdout.
             step (int): Print progress every ``step`` epochs.
             autosave (bool): Save the model with the highest loss to disk.
+            minloss (list[float]): Internal use only.
 
         Raises:
             SystemExit: If ``batch`` is invalid.
     """
     # keep data on device and in a consistent dtype
-    X = cp.asarray(input, dtype=self.dtype)
-    Y = cp.asarray(output, dtype=self.dtype)
+    X = cp.array(input, dtype=self.dtype)
+    Y = cp.array(output, dtype=self.dtype)
 
     # device-side bias column (avoid NumPy)
     bias = cp.ones((X.shape[0], 1), dtype=self.dtype)
     X = cp.concatenate((bias, X), axis=1)
 
     io = IO()
-    minloss = 99999999
-    def progress(epoch, h, y):
+    def progress(yhat, y):
         nonlocal minloss
-        loss = self.loss(yhat=h, y=y)
+        loss = self.loss(y=y, yhat=yhat)
         loss = cp.mean(loss)
-        if loss < minloss:
-            minloss = loss
+        if loss < minloss[0]:
+            minloss[0] = loss
             if autosave:
                 io.save(self, self.backup)
         return loss
 
     n = X.shape[0]
-    if batch == 1:
-      for epoch in range(epochs):
-        index = random.randint(0, n - 1)
+    if batch == 0:
+        yhat = self.batch(X, Y)
+    elif batch == 1:
+      indices = list(range(n))
+      random.shuffle(indices)
+      yhat = cp.zeros(Y.shape)
+      for index in indices:
         h = self.batch(X[index], Y[index])
-        if epoch % step == 0:
-            prog = progress(epoch, h, Y[index])
-            if verbose:
-                logging.info("epoch %d loss %.8f" % (epoch, prog))
-    elif batch == 0:
-      for epoch in range(epochs):
-        h = self.batch(X, Y)
-        if epoch % step == 0:
-            prog = progress(epoch, h, Y)
-            if verbose:
-                logging.info("epoch %d loss %.8f" % (epoch, prog))
+        yhat[index] = h.ravel()
     elif 1 < batch < n:
-      for epoch in range(epochs):
-        index = random.randint(0, n - batch)
+      indices = list(range(0,n,batch))
+      random.shuffle(indices)
+      yhat = cp.zeros(Y.shape)
+      for index in indices:
+        if index + batch > n:
+            continue
         x = X[index:index + batch]
         y = Y[index:index + batch]
         h = self.batch(x, y)
-        if epoch % step == 0:
-            prog = progress(epoch, h, y)
-            if verbose:
-                logging.info("epoch %d loss %.8f" % (epoch, prog))
+        yhat[index:index + batch] = h.ravel()
     else:
       sys.exit(f"improper batch size {batch}")
 
+    if epoch % step == 0:
+        prog = progress(yhat, Y)
+        if verbose:
+            logging.info("epoch %d loss %.8f" % (epoch, prog))
+
+    if self.scheduler is not None:
+      self.lr = self.scheduler.step(epoch, epochs)
+
   def batch(self, x, y):
     """Run a single forward/backward/update step.
 
@@ -153,7 +163,7 @@ class NN(object):
             y (cupy.ndarray | numpy.ndarray): Targets, shape (B, K) or (K,).
 
         Returns:
-            h: the predictions as a tensor
+            q: the predictions as a tensor
     """
     # ensure inputs reside on device & 2D
     x = cp.nan_to_num(cp.atleast_2d(cp.asarray(x)), nan=0.0)
@@ -177,13 +187,13 @@ class NN(object):
     prev_grad = None
     for i in range(len(self.architecture) - 1, -1, -1):
       if i == len(self.architecture) - 1:
-        loss_grad = cp.nan_to_num(self.loss_derivative(yhat=q, y=y), nan=0.0)
+        loss_grad = cp.nan_to_num(self.loss_derivative(y=y, yhat=q), nan=0.0)
         grad = cp.nan_to_num(loss_grad * self.resolver_derivative(raw_outputs[i]), nan=0.0)
       else:
         grad = cp.nan_to_num((prev_grad @ self.W[i + 1].T) * self.activation_derivative(raw_outputs[i]), nan=0.0)
 
       # in-place weight update
-      self.W[i] -= self.learning_rate * (inputs[i].T @ grad)
+      self.W[i] -= self.lr * (inputs[i].T @ grad)
       self.W[i] = cp.nan_to_num(self.W[i], nan=0.0)
       prev_grad = grad
 
@@ -199,18 +209,17 @@ class NN(object):
         Returns:
             cupy.ndarray: Model outputs of shape (n_samples, n_outputs).
     """
-    x = cp.atleast_2d(cp.asarray(input))
-    bias = cp.ones((x.shape[0], 1), dtype=x.dtype)
-    x = cp.concatenate((bias, x), axis=1)
+    yhat = cp.atleast_2d(cp.asarray(input))
+    bias = cp.ones((yhat.shape[0], 1), dtype=yhat.dtype)
+    yhat = cp.concatenate((bias, yhat), axis=1)
 
-    q = x
     for i in range(len(self.architecture)):
-      z = cp.nan_to_num(q @ self.W[i], nan=0.0)
+      h = cp.nan_to_num(yhat @ self.W[i], nan=0.0)
       if i == len(self.architecture) - 1:
-        q = cp.nan_to_num(self.resolver(z), nan=0.0)
+        yhat = cp.nan_to_num(self.resolver(h), nan=0.0)
       else:
-        q = cp.nan_to_num(self.activation(z), nan=0.0)
-    return q
+        yhat = cp.nan_to_num(self.activation(h), nan=0.0)
+    return yhat
 
   def summary(self):
     """Print layer shapes and total parameter count."""
