@@ -2,6 +2,8 @@ from nillanet.model import NN
 from nillanet.activations import Activations
 from nillanet.loss import Loss
 from nillanet.distributions import Distributions
+from nillanet.scheduler import Scheduler
+from nillanet.initializer import Initializer
 
 d = Distributions()
 x,y = d.logical_distribution(10,"and")
@@ -23,20 +25,19 @@ output = y
 features = x.shape[1]
 architecture = [4,8,1]
 learning_rate = 0.01
-epochs = 1000
+epochs = 10000
+scheduler = Scheduler(mode='cosine', lr=learning_rate)
+initializer = Initializer(distribution=Initializer.he, low=0, high=1, mean=0, std=0.5)
+
+model = NN(features,architecture,activation,derivative1,resolver,derivative2,loss,derivative3,learning_rate,scheduler=scheduler,initializer=initializer)
+model.summary()
+
+for epoch in range(epochs):
+    model.train(input,output,epoch,epochs,verbose=True,step=100,autosave=True)
 
-model = NN(features,architecture,activation,derivative1,resolver,derivative2,loss,derivative3,learning_rate)
-model.train(input,output,epochs,verbose=True,step=100,autosave=True)
 prediction = model.predict(x)
 
 print("prediction")
 print(prediction)
 print("expected")
 print(y)
-
-from nillanet.io import IO
-io = IO()
-best = io.load(model.backup)
-prediction = best.predict(x)
-print("best")
-print(prediction)
\ No newline at end of file
